@@ -6,6 +6,7 @@ Swagger pre-processor
 from markdown import util
 from markdown.preprocessors import Preprocessor
 from markdown.extensions import Extension
+import yaml
 import json
 import re
 
@@ -31,14 +32,17 @@ def idRepr(path):
 
 class SwaggerDefinition():
 
-    def __init__(self, file=None, definitionsUrl='', definitionNames=[]):
+    def __init__(self, file=None, definitionsUrl='', definitionNames=[], config={}):
         self.defaultFile = file
         self.definitionsUrl = definitionsUrl
         self.definitionName = None
         self.definitionNames = definitionNames
-
         self.excludeField = ['type', 'items', 'properties', 'required', '$ref', 'xml', 'format', 'name']
 
+        self.config = {
+            "properties": config.get("properties", {})
+        }
+
     def getDefinitionName(self, line):
         content = line.split(' ')
         return content[-1]
@@ -69,17 +73,29 @@ class SwaggerDefinition():
         </table>
         """
 
+    def propetyConfig(self, prop, name):
+        config = self.config['properties']
+        if prop in config and name in config[prop]:
+            return config[prop][name]
+        
+
     def definitionTable(self, definition, defname):
         body = []
         required = definition.get('required', [])
         properties = definition.get('properties', {})
         for name, content in properties.items():
+            if self.propetyConfig(name, 'hide') == True:
+                continue
             self.addTableLine([defname], body, name, content, required)
         return self.table(body=''.join(body), id=defname)
 
-    def details(self, content):
+    def details(self, content, name):
         out = []
         keys = content.keys()
+        description = self.propetyConfig(name, 'description')
+        if description:
+            content['description'] = description
+
         for detail in keys:
             if detail not in self.excludeField:
                 labelValue(out, content, detail)
@@ -107,9 +123,9 @@ class SwaggerDefinition():
 
         return f'<a href="{url}">{name}</a>' 
 
-    def addTableLine(self, path, body, name, content, required=[]):
+    def addTableLine(self, path, body: list, name, content, required=[]):
         ctype = self.typeAndFormat(content)
-        details = self.details(content)
+        details = self.details(content, name)
         ctypeOut = ctype
 
         # could create issue if the name clash...
@@ -143,13 +159,24 @@ class SwaggerDefinition():
 
 class SwaggerPath():
 
-    def __init__(self, file=None, definitionsUrl='', definitionNames=[]):
+    def __init__(self, file=None, definitionsUrl='', definitionNames=[], config={}):
         self.defaultFile = file
         self.definitionsUrl = definitionsUrl
         self.definitionNames = definitionNames
 
         self.excludeField = ['type', 'items', 'properties', 'required', '$ref', 'xml', 'schema', 'format', 'name']
 
+        sectionConfig = config.get("sections", {})
+    
+        self.config = {
+            "responseExamples": sectionConfig.get("responseExamples", True),
+            "responseTable": sectionConfig.get("responseTable", True),
+            "requestExamples": sectionConfig.get("requestExamples", True),
+            "requestCodeExamples": sectionConfig.get("requestCodeExamples", True), 
+            "parametersTable": sectionConfig.get("parametersTable", True),
+            "verbs": config.get("verbs", "all")
+        }
+
     # Typical input
     # :swg-path: /my-project"
     # :swg-path: test_swagger.json /users/{userId}
@@ -170,6 +197,9 @@ class SwaggerPath():
         out = []
         verbs = pathDef.keys()
         for verb in verbs:
+            if self.config['verbs'] != 'all' and not verb in self.config['verbs']:
+                continue
+
             out.append(f'''<p class="sw-path">
                 <span class="sw-verb">{verb.upper()}</span>
                 <span class="sw-path-url">{self.path}</span></p>''')
@@ -177,11 +207,22 @@ class SwaggerPath():
             summary = verbDef.get('summary')
             out.append(f'''<p class="sw-summary">{summary}</p>''')
             parameters = verbDef.get('parameters')
-            out.append(self.parameters(parameters))
 
-            out.append(self.responses(verbDef))
+            if self.config['parametersTable']:
+                out.append(self.parameters(parameters))
+
+            if self.config['requestExamples']:
+                out.append(self.requestExamples(verbDef))
+
+            if self.config['requestCodeExamples']:
+                out.append(self.requestCodeExamples(verb, pathDef, verbDef))
+
+            if self.config['responseTable']:
+                out.append(self.responses(verbDef))
+
+            if self.config['responseExamples']:
+                out.append(self.responsesExamples(verbDef))
 
-            out.append(self.responsesExamples(verbDef))
 
         return '\n'.join(out)
 
@@ -220,6 +261,42 @@ Response example {name}
         return '\n'.join(out)
 
 
+    def requestExamples(self, verbDef):
+        obj = self.requestParameters(verbDef)
+        if not len(obj.keys()):
+            return ''
+        return f'''
+Request example
+
+```json
+{json.dumps(obj, indent=2)}
+```
+'''
+
+    def requestCodeExamples(self, verb, pathDef, verbDef):
+        obj = self.requestParameters(verbDef)
+        if not len(obj.keys()):
+            return ''
+        scheme = self.data.get('schemes', ['https'])[0]
+        host = self.data.get('host', 'example.com')
+        consumes = verbDef.get('consumes', ['application/json'])[0]
+        data = json.dumps(obj, indent=2).replace("\n", "\\\n")
+
+        code = f'''curl {scheme}://{host}{self.path} \\
+--header "Content-Type: {consumes}" \\
+--request {verb.upper()} \\
+--data '{data}'
+'''
+
+        out = f'''
+Request code example
+
+```bash
+{code}
+```
+'''
+        return out
+
     def response(self, name, response):
         description = response.get('description')
         schema = self.contentType(response) or ''
@@ -317,18 +394,41 @@ Response example {name}
     
         return self.parametersTable(''.join(out))
 
-    #     "schema":{
-    #         "$ref":"#/definitions/Pet"
-    #     }
+    def requestParameters(self, verbDef):
+        out = {}
+        for p in verbDef.get('parameters', []):
+            out[p['name']] = self.requestMap(p)
+    
+        return out
+    
+    def requestMap(self, content):
+        name = content.get('name')
+
+        ctype = content.get('type') or content.get('schema', {}).get('type')
+        schema = content.get('schema', {})
+        ref = content.get('$ref') or schema.get('$ref')
+
+        if ctype == 'array':
+            items = content.get('items') or content.get('schema', {}).get('items')
+            return [self.requestMap(items)]
+
+        elif ctype == 'object':
+            properties = content.get('properties', {})
+            c = {}
+            for name, ct in properties.items():
+                c[name] = self.requestMap(ct)
+            return c
+        elif ref:
+            defName = ref.split('#/definitions/')[1]
+            if defName in self.data['definitions']:
+                return self.requestMap(
+                    self.data['definitions'][defName])
+        else:
+            return self.getRandomValue(content)
 
-    # "schema":{
-    #     "type":"array",
-    #     "items":{
-    #     "$ref":"#/definitions/Pet"
-    #     }
-    # }
     def getRandomValue(self, content):
         ctype = content.get('type')
+        format = content.get('format')
         example = content.get('example')
         if example:
             return example
@@ -336,15 +436,20 @@ Response example {name}
         if enum:
             return enum[0]
 
-        if ctype == 'integer':
+        if ctype == 'integer' or ctype == 'number':
             return 123
         elif ctype == 'string':
-            return 'some string'
+            if format == 'date':
+                return '2019-07-21'
+            if format == 'date-time':
+                return '2017-07-21T17:32:28Z'
+            if format == 'password':
+                return '*****'
+            return 'lorem ipsum'
         elif ctype == 'boolean':
             return True
         
-        return '?'
-
+        return ctype
 
     def responseMap(self, content):
         name = content.get('name')
@@ -369,6 +474,7 @@ Response example {name}
                     self.data['definitions'][defName])
         else:
             return self.getRandomValue(content)
+
 class SwaggerPreprocessor(Preprocessor):
     """Swagger include Preprocessor"""
 
@@ -377,6 +483,19 @@ class SwaggerPreprocessor(Preprocessor):
         self.definitionsUrl = definitionsUrl
         super(SwaggerPreprocessor, self).__init__(md)
 
+    def getConfig(self, index, lines):
+        yamlOut = []
+        while index < len(lines):
+            line = lines[index]
+            if line.startswith('    '):
+                yamlOut.append(line[4:])
+                del lines[index]
+            else:
+                break
+        if len(yamlOut):
+            return yaml.load('\n'.join(yamlOut), Loader=yaml.FullLoader)
+        return {}
+
     def run(self, lines):
         out = []
 
@@ -384,23 +503,27 @@ class SwaggerPreprocessor(Preprocessor):
         definitionNames = []
         for line in lines:
             if line.startswith(':swg-def: '):
-              definitionNames.append(SwaggerDefinition().getDefinitionName(line))
+                definitionNames.append(SwaggerDefinition().getDefinitionName(line))
 
-        for line in lines:
+        for index, line in enumerate(lines):
             if line.startswith(':swg-def: '):
-              handler = SwaggerDefinition(
-                file=self.defaultFile, 
-                definitionsUrl=self.definitionsUrl,
-                definitionNames=definitionNames
-              )
-              out = out + handler.handleLine(line).split("\n")
+                lineConfig = self.getConfig(index + 1, lines)
+                handler = SwaggerDefinition(
+                    file=self.defaultFile, 
+                    definitionsUrl=self.definitionsUrl,
+                    definitionNames=definitionNames,
+                    config=lineConfig
+                )
+                out = out + handler.handleLine(line).split("\n")
             elif line.startswith(':swg-path: '):
-              handler = SwaggerPath(
-                file=self.defaultFile,
-                definitionsUrl=self.definitionsUrl,
-                definitionNames=definitionNames
-              )
-              out = out + handler.handleLine(line).split("\n")
+                lineConfig = self.getConfig(index + 1, lines)
+                handler = SwaggerPath(
+                    file=self.defaultFile,
+                    definitionsUrl=self.definitionsUrl,
+                    definitionNames=definitionNames,
+                    config=lineConfig
+                )
+                out = out + handler.handleLine(line).split("\n")
             else:
               out.append(line)
 
